


#include <tqcstring.h>
#include <tqimage.h>
#include <tqpainter.h>
#include <tqpalette.h>
#include <tqpixmap.h>
#include <tqfont.h>

#include <kjs/object.h>

#include <kjsembed/global.h>
#include <kjsembed/jsobjectproxy.h>
#include <kjsembed/jsopaqueproxy.h>
#include <kjsembed/jsbinding.h>

#include <tqcanvas.h>
#include "qcanvasspline_imp.h"

/**
 * Namespace containing the KJSEmbed library.
 */
namespace KJSEmbed {

TQCanvasSplineImp::TQCanvasSplineImp( KJS::ExecState *exec, int mid, bool constructor )
   : JSProxyImp(exec), id(mid), cons(constructor)
{
}

TQCanvasSplineImp::~TQCanvasSplineImp()
{
}

/**
 * Adds bindings for static methods and enum constants to the specified Object.
 */
void TQCanvasSplineImp::addStaticBindings( KJS::ExecState *exec, KJS::Object &object )
{
    JSProxy::MethodTable methods[] = {

	{ 0, 0 }
    };

    int idx = 0;
    TQCString lastName;

    while( methods[idx].name ) {
        if ( lastName != methods[idx].name ) {
            TQCanvasSplineImp *meth = new TQCanvasSplineImp( exec, methods[idx].id );
            object.put( exec , methods[idx].name, KJS::Object(meth) );
            lastName = methods[idx].name;
        }
        ++idx;
    }


}

/**
 * Adds bindings for instance methods to the specified Object.
 */
void TQCanvasSplineImp::addBindings( KJS::ExecState *exec, KJS::Object &object )
{
    JSProxy::MethodTable methods[] = {

        { Method_setControlPoints_3, "setControlPoints" },
        { Method_controlPoints_4, "controlPoints" },
        { Method_closed_5, "closed" },
        { Method_rtti_6, "rtti" },
	{ 0, 0 }
    };

    int idx = 0;
    TQCString lastName;

    while( methods[idx].name ) {
        if ( lastName != methods[idx].name ) {
            TQCanvasSplineImp *meth = new TQCanvasSplineImp( exec, methods[idx].id );
            object.put( exec , methods[idx].name, KJS::Object(meth) );
            lastName = methods[idx].name;
        }
        ++idx;
    }
}

/**
 * Extract a TQCanvasSpline pointer from an Object.
 */
TQCanvasSpline *TQCanvasSplineImp::toTQCanvasSpline( KJS::Object &self )
{
    JSObjectProxy *ob = JSProxy::toObjectProxy( self.imp() );
    if ( ob ) {
        TQObject *obj = ob->object();
	if ( obj )
           return dynamic_cast<TQCanvasSpline *>( obj );
    }

    JSOpaqueProxy *op = JSProxy::toOpaqueProxy( self.imp() );
    if ( !op )
        return 0;

    if ( op->typeName() != "TQCanvasSpline" )
        return 0;

    return op->toNative<TQCanvasSpline>();
}

/**
 * Select and invoke the correct constructor.
 */
KJS::Object TQCanvasSplineImp::construct( KJS::ExecState *exec, const KJS::List &args )
{
   switch( id ) {

         case Constructor_QCanvasSpline_1:
             return TQCanvasSpline_1( exec, args );
             break;

         default:
             break;
    }

    TQString msg = i18n("TQCanvasSplineCons has no constructor with id '%1'.").arg(id);
    return throwError(exec, msg,KJS::ReferenceError);
}


KJS::Object TQCanvasSplineImp::TQCanvasSpline_1( KJS::ExecState *exec, const KJS::List &args )
{

    // Unsupported parameter TQCanvas *
    return KJS::Object();

    TQCanvas * arg0; // Dummy


    // We should now create an instance of the TQCanvasSpline object

    TQCanvasSpline *ret = new TQCanvasSpline(

          arg0 );


}

KJS::Value TQCanvasSplineImp::call( KJS::ExecState *exec, KJS::Object &self, const KJS::List &args )
{
    instance = TQCanvasSplineImp::toTQCanvasSpline( self );

    switch( id ) {

    case Method_setControlPoints_3:
        return setControlPoints_3( exec, self, args );
        break;

    case Method_controlPoints_4:
        return controlPoints_4( exec, self, args );
        break;

    case Method_closed_5:
        return closed_5( exec, self, args );
        break;

    case Method_rtti_6:
        return rtti_6( exec, self, args );
        break;

    default:
        break;
    }

    TQString msg = i18n( "TQCanvasSplineImp has no method with id '%1'." ).arg( id );
    return throwError(exec, msg,KJS::ReferenceError);
}


KJS::Value TQCanvasSplineImp::setControlPoints_3( KJS::ExecState *exec, KJS::Object &obj, const KJS::List &args )
{

    // Unsupported parameter TQPointArray
    return KJS::Value();

    TQPointArray arg0; // Dummy

    bool arg1 = extractBool(exec, args, 1);

      instance->setControlPoints(
       arg0,
       arg1 );
      return KJS::Value(); // Returns void

}

KJS::Value TQCanvasSplineImp::controlPoints_4( KJS::ExecState *exec, KJS::Object &obj, const KJS::List &args )
{

      instance->controlPoints(  );
      return KJS::Value(); // Returns 'TQPointArray'

}

KJS::Value TQCanvasSplineImp::closed_5( KJS::ExecState *exec, KJS::Object &obj, const KJS::List &args )
{

      bool ret;
      ret = instance->closed(  );
      return KJS::Boolean( ret );

}

KJS::Value TQCanvasSplineImp::rtti_6( KJS::ExecState *exec, KJS::Object &obj, const KJS::List &args )
{

      int ret;
      ret = instance->rtti(  );
      return KJS::Number( ret );

}


} // namespace KJSEmbed
